classdef Elevator < handle
properties
    C;  % max occupancy
    D;  % distance between floors
    K;  % max jerk
    
    passengers = [];  % array of passengers on the elevator
    w = cell(3,1);  % arrays of waiting passengers
    exited = [];  % passengers that have exited the elevator

    interior = [0 0 0];  % interior elevator buttors 0 = off, 1 = on
    exterior = [0 0 0 0];  % button 1 = 1st floor up, 2 = 2nd floor down
                           % 3 = 2nd floor up, 3 = 3rd floor down
                
    current = 1;  % elevator starts on floor 1  
    last = 1;  % previous floor
    next = 0;  % 0 means the elevator is currently inactive
end
methods
    % Constructor for the Elevator. Initializes variable parameters.
    function this=Elevator(occupancy, distance, jerk)
        this.K = jerk;
        this.C = occupancy;
        this.D = distance;
    end

    % Called when an elevator arrives at the floor given by this.next.
    function arrive(this)
        global EVENT_QUEUE;
        
        this.last = this.current;
        this.current = this.next;

        % exit all passengers bound for this floor
        
        % board people waiting at this floor
        
        this.computeLights();
        % Determine the next floor.
    end

    % Called to add a passenger to the elevator waiting area. Determines
    % which area based on the source field of the passenger.
    function addPassenger(this, p)
        this.w{p.source} = [this.w{p.source} p];
        this.computeLights();
        this.wake();
    end

    % Call this every time the passengers and queues change to keep
    % the state of the buttons up to date.
    function computeLights(this)
    end
    
    % The elevator goes into a sleep mode when there are no passengers.
    % Call this function in order to bring it out of sleep mode when
    % new passengers arrive. Does nothing if the elevator is not asleep,
    % so it is safe to call often.
    function wake(this)
        if (this.next == 0)
            this.computeNext();
            if (this.next ~= 0)
                this.scheduleNext();
            end
        end
    end
    
    % Assumes that the elevator is not resting (next is nonzero). Schedules
    % an event to bring the elevator to the next floor based on how long it
    % should take.
    function scheduleNext(this)
        global EVENT_QUEUE;
        if (this.next == this.current)
            this.arrive();
        else
            t = EVENT_QUEUE.currentTime + this.computeTravelTime(abs(this.next-this.current));
            EVENT_QUEUE.enqueue(Event(t,'elevatorArrive',struct('elevator',this)));
        end
    end
    
    % Computes time based on the distance and the max jerk
    function t=computeTravelTime(this, numFloors)
    end

    % A helper function that reduces the exterior buttons to 3 by combining
    % the up/down buttons of floor 2.
    function ext = exteriorReduced(this)
        ext = [this.exterior(1) this.exterior(2)||this.exterior(3) this.exterior(4)];
    end
    
    % This example logic computes the next floor randomly. If there are
    % multiple possible next floors, it chooses randomly between the two.
    function computeNext(this)
        % If all buttons are inactive, then rest at current floor
        if (sum(this.interior) + sum(this.exterior) == 0)
            this.next = 0;
            return;
        end
        % If any buttons on other floors are active
        active = this.interior | this.exteriorReduced();
        if (sum(active) - active(this.current) > 0)
            % If there is activity on both other floors
            if (sum(active) - active(this.current) >= 2)
                % Randomly choose one of the floors to service.
                this.next = mod(this.current-1+random('unid', 2), 3)+1;
            else
                % Otherwise there is only one requested floor ~= current
                for floor = 1:3
                    if (floor == this.current)
                        continue
                    elseif (active(floor))
                        this.next = floor;
                    end
                end
            end
        else
            % If only buttons on the current floor are active
            this.next = this.current;
        end
    end
end
end