classdef EventQueue < handle
    properties
        queue
        currentTime
    end
    
    methods
        function this=EventQueue()
            this.queue = [];
            this.currentTime = 0;
        end
        
        % Runs events sequentially as long as the event queue is
        % nonempty and time is less than the next scheduled event.
        function rununtil(this, time)
           while (~isempty(this.queue) && this.currentTime < time)
               this.step();
           end
        end
        
        % Causes the next event in the event queue to be executed.
        function step(this)
            minIdx = 1;
            minTime = inf;
            for idx = 1:length(this.queue)
                ev = this.queue(idx);
                if (ev.timestamp < minTime)
                    minTime = ev.timestamp;
                    minIdx = idx;
                end
            end
            ev = this.queue(minIdx);
            this.queue(minIdx) = [];
            this.currentTime = minTime;
            ev.execute(); 
        end
        
        % Schedules an event for execution in the event queue.
        function enqueue(this, ev)
            this.queue = [this.queue ev];
        end
    end
end